#ifndef LIBPARSE_H
#define LIBPARSE_H

#include "UString"
#include "UHash"
#include "UControl"
#include "LibError.h"

class LibGroup;
class LibDefine;

typedef UHashMap<UString, LibDefine*> LibDefineHashMap;
typedef void (*LibMsgHandlerT) (LibSeverityT sev, LibErrorT errToPrint,
				const char* auxText);

class LibParse
{
public:
  static void debugMode(uint_t i) { d_debugMode = i; }
  static bool debugMode() { return d_debugMode; }

  static LibGroup* group() { return d_group; }

  static void msgHandler(LibMsgHandlerT mh) { d_msgHandler = mh; }
  static LibMsgHandlerT msgHandler() { return d_msgHandler; }

  static void debug(LibSeverityT sev, LibErrorT err, const char* auxText);
  static void error(LibSeverityT sev, LibErrorT err, const char* auxText);

  static void init(const UControl&);
  static void quit();

  static void addToDefineHash(const UString& name, LibDefine *d);
  static LibDefine* findDefine(const UString& name);

  static LibErrorT read(const char* filename);
  static void write(const char* filename, LibGroup* group);
  static LibErrorT checkSyntax(LibGroup* lib);

  static void LibDefaultMsgHandler(LibSeverityT sev, LibErrorT err,
				   const char* auxText);

  static const char* filename() {return d_filename;}
  static const UControl* control() {return d_control;}

private:
  static LibGroup*            d_group;
  static LibDefineHashMap*    d_defineHash;
  static LibMsgHandlerT       d_msgHandler;
  static uint_t               d_debugMode;
  static const char*          d_filename;
  static const UControl*      d_control;
};

inline void
LibParse::debug(LibSeverityT sev, LibErrorT err, const char* auxText)
{
  if (d_debugMode) {
    (*d_msgHandler) (sev, err, auxText);
  }
}

inline void
LibParse::error(LibSeverityT sev, LibErrorT err, const char* auxText)
{
  (*d_msgHandler) (sev, err, auxText);
}

inline void
LibParse::addToDefineHash(const UString& name, LibDefine *d)
{
  d_defineHash->quickAdd(name).value(d);
}

inline LibDefine*
LibParse::findDefine(const UString& name)
{
  LibDefineHashMap::iterator iter;
  if ((iter = d_defineHash->find(name))) {
    return iter.value();
  } else {
    return NULL;
  }
}

#endif

