/******************************************************************************
    Copyright (c) 1996-2000 Synopsys, Inc.    ALL RIGHTS RESERVED

  The contents of this file are subject to the restrictions and limitations
  set forth in the SYNOPSYS Open Source License Version 1.0  (the "License"); 
  you may not use this file except in compliance with such restrictions 
  and limitations. You may obtain instructions on how to receive a copy of 
  the License at

  http://www.synopsys.com/partners/tapin/tapinprogram.html. 

  Software distributed by Original Contributor under the License is 
  distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either 
  expressed or implied. See the License for the specific language governing 
  rights and limitations under the License.

******************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "UMem"
#include "mymalloc.h"

static size_t my_malloc_totals = 0;
static int calls_calloc = 0, calls_malloc = 0, calls_realloc = 0;

#ifdef LIBERTY_USE_MEMPOOL
UMemPool *liberty_pool = NULL;
#endif

void
liberty_pool_init ()
{
#ifdef LIBERTY_USE_MEMPOOL
  if (!liberty_pool) {
    liberty_pool = new UMemPool("liberty");
  }
#endif
}

void
liberty_pool_free ()
{
#ifdef LIBERTY_USE_MEMPOOL
  liberty_pool->cleanup();
  delete liberty_pool;
  liberty_pool = NULL;
#endif
}

void *
my_malloc(int size)
{
  void *ret;

  calls_malloc++;
  my_malloc_totals += (size_t)size;

#ifdef LIBERTY_USE_MEMPOOL
  ret = liberty_pool->newMem((size_t)size);
#else
  ret = malloc((size_t)size);
#endif
  if (!ret) {
    printf("Failure to fetch %d bytes of memory!\n", size);
  }
  return ret;
}

void
my_free(void *ptr)
{
  /* do all the freeing in one big burp later */
#ifdef LIBERTY_USE_MEMPOOL
  if (ptr) {
    UMem::delMem(ptr);
  }
#else
  if (ptr) {
    free(ptr);
  }
#endif
}


void *
my_calloc(int size, int numels)
{
  void *ret;
  size_t total_size = (size_t)(size * numels);

  calls_calloc++;
  my_malloc_totals += total_size;
#ifdef LIBERTY_USE_MEMPOOL
  ret = liberty_pool->newMem(total_size);
#else
  ret = calloc((size_t)size, (size_t)numels);
#endif
  if (!ret) {
    printf("Failure to fetch %d bytes of memory!\n", size * numels);
#ifdef LIBERTY_USE_MEMPOOL
  } else {
    memset (ret, 0, total_size);
#endif
  }
  return ret;
}

void *
my_realloc(char *ptr, int size)
{
  void *ret;

  calls_realloc++;

  my_malloc_totals += (size_t)size;

#ifdef LIBERTY_USE_MEMPOOL
  ret = uRealloc((void*)ptr, (size_t)size);
#else
  ret = realloc((void *)ptr, (size_t)size);
#endif

  if (!ret) {
    printf("Failure to fetch %d bytes of memory!\n", size);
  }
  return ret;
}

void
my_malloc_print_totals(void)
{
  printf("Totals:\n Calls to malloc = %d\n Calls to calloc = %d\n\n",
	 calls_malloc, calls_calloc);
  printf(" Calls to realloc = %d\n", calls_realloc);
}
